import logging
import json

from pyIndego import IndegoClient

logging.basicConfig(filename="pyIndego.log", level=logging.DEBUG)
_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)


def main(config):
    with IndegoClient(**config) as indego:
        # print("=[alerts]====")
        indego.update_state()
        print(indego.state)
        indego.update_state(longpoll=True, longpoll_timeout=20)
        print(indego.state)
        indego.update_alerts()
        print(indego.alerts)
        # print("=[alerts_count]====")
        # print(indego.alerts_count)

        # print("=[generic_data]====")
        # indego.update_generic_data()
        # print(indego.generic_data)

        # print("=[last_completed_mow]====")
        # indego.update_last_completed_mow()
        # print(indego.last_completed_mow)

        # print("=[location]====")
        # indego.update_location()
        # print(indego.location)

        # print("=[longpoll_state]====")
        # indego.update_longpoll_state(40)
        # print(indego.longpoll_state)

        # indego.update_network()
        # print(indego.network)
        # indego.update_all(force=True)
        # print(indego)
        # print("=[state]====")
        # indego.update_state()
        # print(indego.state)
        
        print(" ")
        print("=[alerts]====")
        indego.update_alerts()
        print(indego.alerts)
        print("=[alerts_count]====")
        print(indego.alerts_count)

        print(" ")
        print("=[generic_data]====")
        indego.update_generic_data()
        print(indego.generic_data)

        print(" ")
        print("=[last_completed_mow]====")
        indego.update_last_completed_mow()
        print(indego.last_completed_mow)

        print(" ")
        print("=[location]====")
        indego.update_location()
        print(indego.location)

        # print("=[longpoll_state]====")
        # indego.update_longpoll_state(40)
        # print(indego.longpoll_state)

        print(" ")
        print("=[operating data]====")
        indego.update_operating_data()
        print(indego.operating_data)

        # indego.update_network()
        # print(indego.network)

        # indego.update_all(force=True)
        # print(indego)

        print("=[state]====")
        indego.update_state()
        print(indego.state)

        # indego.update_next_mow()
        # indego.update_operating_data()
        # # indego.update_updates()
        # # indego.update_users()
        # # indego.update_network()
        
        print("=[updates]====")
        indego.update_updates_available()
        print(indego.update_available)
        print(" ")
        print("=[users]====")
        indego.update_users()
        print(indego.users)
        # indego.update_network()

        # indego.download_map()

        # print("map: ", indego.map_filename)
        # print("network ", indego.network)
        # print("state: ", indego.state)
        # print("users: ", indego.users)
        # print("Generic data: ", indego.generic_data)
        # print("Generic data min voltage: ", indego.generic_data.model_voltage.min)
        # print("Alerts: ", indego.alerts)
        # print("Operating_data: ", indego.operating_data)
        # print("Battery: ", indego.operating_data.battery)
        # print("Battery percent: ", indego.operating_data.battery.percent_adjusted)
        # print("Battery voltage: ", indego.operating_data.battery.voltage)
        # # indego.update_generic_data()
        # print("Battery: ", indego.operating_data.battery)
        # print("Session charge: ", indego.operating_data.runtime.session.charge)

        # print("Next mow: ", indego.next_mow)
        print(" ")
        indego.update_next_mow()
        print("Next mow: ", indego.next_mow)

        # print("location: ", indego.location)
        # print("last mow: ", indego.last_completed_mow)
        # print(indego.generic_data.alm_mode)
        # print(indego.alm_name)
        # print(indego.service_counter)
        # print(indego.needs_service)
        # print(indego.alm_mode)
        # print(indego.bare_tool_number)
        # print(indego.alm_firmware_version)
        # print(indego.model_description)
        # print(indego.model_voltage)
        # print(indego.mowing_mode_description)
        # print(indego.model_description)
        # print(indego.model_description)
        # print(indego.mower_state)
        # print(indego.mower_state_description)
        # print(indego.mower_state_description_detailed)
        # print(indego.battery)
        # print(indego.email)
        # print(indego.garden)
        # print(indego.x_pos, ", ", indego.y_pos)
        # indego.show_vars()


if __name__ == "__main__":
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
    main(config)



from enum import Enum


class Methods(Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"


DEFAULT_URL = "https://api.indego.iot.bosch-si.com:443/api/v1/"
# CONST TAKEN FROM homeassistant.const
CONTENT_TYPE_JSON = "application/json"
# const taken from aiohttp.hdrs
CONTENT_TYPE = "Content-Type"
DEFAULT_BODY = {
    "device": "",
    "os_type": "Android",
    "os_version": "4.0",
    "dvc_manuf": "unknown",
    "dvc_type": "unknown",
}
COMMANDS = ("mow", "pause", "returnToDock")

DEFAULT_HEADER = {CONTENT_TYPE: CONTENT_TYPE_JSON}
DEFAULT_LOOKUP_VALUE = "Not in database."

DEFAULT_CALENDAR = {
    "sel_cal": 1,
    "cals": [
        {
            "cal": 1,
            "days": [
                {
                    "day": 0,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 1,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 2,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 3,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 4,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 5,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 8, "EnMin": 0},
                        {"En": True, "StHr": 20, "StMin": 0, "EnHr": 23, "EnMin": 59},
                    ],
                },
                {
                    "day": 6,
                    "slots": [
                        {"En": True, "StHr": 0, "StMin": 0, "EnHr": 23, "EnMin": 59}
                    ],
                },
            ],
        }
    ],
}

MOWER_STATE_DESCRIPTION_DETAIL = {
    "0": "Reading status",
    "101": "Mower lifted",
    "257": "Charging",
    "258": "Docked",
    "259": "Docked - Software update",
    "260": "Charging",
    "261": "Docked",
    "262": "Docked - Loading map",
    "263": "Docked - Saving map",
    "266": "Docked - Leaving dock",
    "512": "Mowing - Leaving dock",
    "513": "Mowing",
    "514": "Mowing - Relocalising",
    "515": "Mowing - Loading map",
    "516": "Mowing - Learning lawn",
    "517": "Mowing - Paused",
    "518": "Border cut",
    "519": "Idle in lawn",
    "520": "Mowing - Learning lawn paused",
    "521": "Border cut",
    "523": "Mowing - Spot mowing",
    "524": "Mowing - Random",
    "525": "Mowing - Random complete",
    "768": "Returning to Dock",
    "769": "Returning to Dock",
    "770": "Returning to Dock",
    "771": "Returning to Dock - Battery low",
    "772": "Returning to dock - Calendar timeslot ended",
    "773": "Returning to dock - Battery temp range",
    "774": "Returning to dock - requested by user/app",
    "775": "Returning to dock - Lawn complete",
    "776": "Returning to dock - Relocalising",
    "1005": "Connection to dockingstation failed",
    "1025": "Diagnostic mode",
    "1026": "End of life",
    "1027": "Service Requesting Status",
    "1038": "Mower immobilized",
    "1281": "Software update",
    "1537": "Stuck on lawn, help needed",
    "64513": "Sleeping",
    "99999": "Offline",
    "None": "None",
}


MOWER_STATE_DESCRIPTION = {
    "0": "Docked",
    "101": "Docked",
    "257": "Docked",
    "258": "Docked",
    "259": "Docked",
    "260": "Docked",
    "261": "Docked",
    "262": "Docked",
    "263": "Docked",
    "266": "Mowing",
    "512": "Mowing",
    "513": "Mowing",
    "514": "Mowing",
    "515": "Mowing",
    "516": "Mowing",
    "517": "Mowing",
    "518": "Mowing",
    "519": "Mowing",
    "520": "Mowing",
    "521": "Mowing",
    "522": "Mowing",
    "523": "Mowing",
    "524": "Mowing",
    "525": "Mowing",
    "768": "Mowing",
    "769": "Mowing",
    "770": "Mowing",
    "771": "Mowing",
    "772": "Mowing",
    "773": "Mowing",
    "774": "Mowing",
    "775": "Mowing",
    "776": "Mowing",
    "1005": "Mowing",
    "1025": "Diagnostic mode",
    "1026": "End of life",
    "1027": "Service Requesting Status",
    "1038": "Mower immobilized",
    "1281": "Software update",
    "1537": "Stuck",
    "64513": "Docked",
    "99999": "Offline",
    "None": "None",
}

MOWER_MODEL_DESCRIPTION = {
    "3600HA2300": "Indego 1000",
    "3600HA2301": "Indego 1200",
    "3600HA2302": "Indego 1100",
    "3600HA2303": "Indego 13C",
    "3600HA2304": "Indego 10C",
    "3600HB0100": "Indego 350",
    "3600HB0101": "Indego 400",
    "3600HB0102": "Indego S+ 350 1gen",
    "3600HB0103": "Indego S+ 400 1gen",
    "3600HB0105": "Indego S+ 350 2gen",
    "3600HB0106": "Indego S+ 400 2gen",
    "3600HB0301": "Indego M+ 700",
}

MOWER_MODEL_VOLTAGE = {
    "3600HA2300": {"min": "297", "max": "369"},  # Indego 1000
    "3600HA2301": {"min": "297", "max": "369"},  # Indego 1200
    "3600HA2302": {"min": "297", "max": "369"},  # Indego 1100
    "3600HA2303": {"min": "297", "max": "369"},  # Indego 13C
    "3600HA2304": {"min": "297", "max": "369"},  # Indego 10C
    "3600HB0100": {"min": "0", "max": "100"},  # Indego 350
    "3600HB0101": {"min": "0", "max": "100"},  # Indego 400
    "3600HB0102": {"min": "0", "max": "100"},  # Indego S+ 350
    "3600HB0103": {"min": "0", "max": "100"},  # Indego S+ 400
    "3600HB0105": {"min": "0", "max": "100"},  # Indego S+ 350
    "3600HB0106": {"min": "0", "max": "100"},  # Indego S+ 400
    "3600HB0301": {"min": "0", "max": "100"},  # Indego M+ 700
}

MOWING_MODE_DESCRIPTION = {
    "smart": "SmartMowing",
    "calendar": "Calendar",
    "manual": "Manual",
}

ALERT_ERROR_CODE = {
    "104": "Stop button pushed",
    "115": "Mower is stuck",
    "149": "Mower outside perimeter cable",
    "151": "Perimeter cable signal missing",
    "ntfy_blade_life": "Reminder blade life",
}



"""Classes for states of pyIndego."""
import logging
import typing
from dataclasses import dataclass
from dataclasses import field
from dataclasses import is_dataclass
from datetime import datetime

from .const import ALERT_ERROR_CODE
from .const import DEFAULT_LOOKUP_VALUE
from .const import MOWER_MODEL_DESCRIPTION
from .const import MOWING_MODE_DESCRIPTION
from .helpers import convert_bosch_datetime
from .helpers import nested_dataclass

_LOGGER = logging.getLogger(__name__)


@dataclass
class Alerts:
    alm_sn: str = field(repr=False, default=None)
    alert_id: str = None
    error_code: str = None
    headline: str = None
    date: datetime = field(default_factory=convert_bosch_datetime)
    message: str = None
    read_status: str = None
    flag: str = None
    push: bool = None
    alert_description: str = None

    def __post_init__(self):
        self.alert_description = ALERT_ERROR_CODE.get(
            self.error_code, DEFAULT_LOOKUP_VALUE
        )


@dataclass
class ModelVoltage:
    min: int = None
    max: int = None


MOWER_MODEL_VOLTAGE = {
    "3600HA2300": ModelVoltage(min=297, max=369),  # Indego 1000
    "3600HA2301": ModelVoltage(min=297, max=369),  # Indego 1200
    "3600HA2302": ModelVoltage(min=297, max=369),  # Indego 1100
    "3600HA2303": ModelVoltage(min=297, max=369),  # Indego 13C
    "3600HA2304": ModelVoltage(min=297, max=369),  # Indego 10C
    "3600HB0100": ModelVoltage(min=0, max=100),  # Indego 350
    "3600HB0101": ModelVoltage(min=0, max=100),  # Indego 400
    "3600HB0102": ModelVoltage(min=0, max=100),  # Indego S+ 350
    "3600HB0103": ModelVoltage(min=0, max=100),  # Indego S+ 400
    "3600HB0105": ModelVoltage(min=0, max=100),  # Indego S+ 350
    "3600HB0106": ModelVoltage(min=0, max=100),  # Indego S+ 400
    "3600HB0301": ModelVoltage(min=0, max=100)  # Indego M+ 700
    #    '3600HB0xxx': {'min': '0','max': '100'}   # Indego M+ 700
}


@dataclass
class Battery:
    percent: int = None
    voltage: float = None
    cycles: int = None
    discharge: float = None
    ambient_temp: int = None
    battery_temp: int = None
    percent_adjusted: int = None

    def update_percent_adjusted(self, voltage: ModelVoltage):
        if self.percent:
            self.percent_adjusted = round(
                (int(self.percent) - voltage.min) / ((voltage.max - voltage.min) / 100)
            )


@nested_dataclass
class GenericData:
    alm_name: str = None
    alm_sn: str = None
    service_counter: int = None
    needs_service: bool = None
    alm_mode: str = None
    bareToolnumber: str = None
    alm_firmware_version: str = None
    model_description: str = None
    model_voltage: ModelVoltage = field(default_factory=ModelVoltage)
    mowing_mode_description: str = None

    def __post_init__(self):
        self.model_description = MOWER_MODEL_DESCRIPTION.get(
            self.bareToolnumber, DEFAULT_LOOKUP_VALUE
        )
        self.model_voltage = MOWER_MODEL_VOLTAGE.get(
            self.bareToolnumber, ModelVoltage()
        )
        self.mowing_mode_description = MOWING_MODE_DESCRIPTION.get(
            self.alm_mode, DEFAULT_LOOKUP_VALUE
        )


@dataclass
class Location:
    latitude: float = None
    longitude: float = None
    timezone: str = None


@dataclass
class Network:
    mcc: int = None
    mnc: int = None
    rssi: int = None
    currMode: str = None
    configMode: str = None
    steeredRssi: int = None
    networkCount: int = None
    networks: typing.List[int] = None

@dataclass
class Config:
    region: int = None
    language: int = None
    border_cut: int = None
    is_pin_set: bool = None
    wire_id: int = None
    bump_sensitivity: int = None
    alarm_mode: bool = None

@dataclass
class RuntimeDetail:
    operate: int = None
    charge: int = None
    cut: int = field(init=False, default=None)

    def update_cut(self):
        # _LOGGER.debug("---Update session cut")
        self.cut = round(self.operate - self.charge)
        # _LOGGER.debug(f"---self.cut = {self.cut}")


@nested_dataclass
class Runtime:
    total: RuntimeDetail = field(default_factory=RuntimeDetail)
    session: RuntimeDetail = field(default_factory=RuntimeDetail)

    def __post_init__(self):
        if self.total.charge:
            # _LOGGER.debug("---self.total.charge")
            self.total.charge = round(self.total.charge / 100)
        if self.total.operate:
            # _LOGGER.debug("---self.total.operate")
            self.total.operate = round(self.total.operate / 100)
        if self.total.charge:
            # _LOGGER.debug("---self.total.charge")
            self.total.update_cut()
        if self.session.charge:
            # _LOGGER.debug("---self.session.charge")
            self.session.update_cut()
        else:
            self.session.cut = 0


@dataclass
class Garden:
    id: int = None
    name: int = None
    signal_id: int = None
    size: int = None
    inner_bounds: int = None
    cuts: int = None
    runtime: int = None
    charge: int = None
    bumps: int = None
    stops: int = None
    last_mow: int = None
    map_cell_size: int = None


@nested_dataclass
class OperatingData:
    hmiKeys: str = None
    battery: Battery = field(default_factory=Battery)
    garden: Garden = field(default_factory=Garden)
    runtime: Runtime = field(default_factory=Runtime)


@nested_dataclass
class State:
    state: int = None
    map_update_available: bool = None
    mowed: int = None
    mowmode: int = None
    xPos: int = None
    yPos: int = None
    charge: int = None
    operate: int = None
    runtime: Runtime = field(default_factory=Runtime)
    mapsvgcache_ts: int = None
    svg_xPos: int = None
    svg_yPos: int = None
    config_change: bool = None
    mow_trig: bool = None


@dataclass
class Users:
    email: str = None
    display_name: str = None
    language: str = None
    country: str = None
    optIn: bool = None
    optInApp: bool = None
